//*******************************************************
//* KWINLIB.H - graphical LIBRARY header file for KLCC.	*
//* Copyright 2005 Everett Kaser						*
//* All rights reserved									*
//*******************************************************

#ifndef _KDEF_DEFINED
#include	<kdef.h>		// include kdef.h if it hasn't been yet
#endif	//_KDEF_DEFINED

#ifndef _KWINLIB_DEFINED	// don't include if already included before
#define	_KWINLIB_DEFINED

#define	DEVCHECKS	1

//***************************************
typedef struct {
		long	x;
		long	y;
		long	w;
		long	h;
		DWORD	osdependant;
	} WNDPOS;

typedef struct {
		long	w;
		long	h;
		short	orient;
	} BMINFO;

typedef struct {
		long	x1;
		long	y1;
		long	x2;
		long	y2;
	} RECT;

typedef struct {
		short	x;
		short	y;
	} POINT;

//***************************************

#define	CLR_NONE	0xFFFFFFFF
#define	CLR_BLACK	0x00000000
#define	CLR_RED		0x000000FF
#define	CLR_GREEN	0x0000FF00
#define	CLR_BLUE	0x00FF0000
#define CLR_YELLOW	0x0000FFFF
#define	CLR_WHITE	0x00FFFFFF
#define	CLR_DEFBACK	0x007B7B00
#define	CLR_DKGRAY	0x00404040
#define	CLR_MDGRAY	0x00808080
#define	CLR_LTGRAY	0x00C0C0C0
#define	CLR_LTGREEN	0x0080FF80

#define CURSOR_NORMAL	0
#define	CURSOR_WAIT		1
#define	CURSOR_LINK		2

#define BM_NOMASK			0
#define BM_NOMASK_INVERT	1
#define BM_MASKED			2
#define BM_MASKED_INVERT	3

#define	BM_STRETCH_PIXEL	0
#define	BM_STRETCH_SMOOTH	1
#define	BM_STRETCH_MASKED	2

#define BMO_NORMAL	0
#define BMO_90		1
#define BMO_180		2
#define BMO_270		3
#define BMO_FNORMAL	4
#define BMO_F90		5
#define BMO_F180	6
#define BMO_F270	7

#define	TS_NORMAL	0
#define	TS_BOLD		2
#define	TS_ITALIC	4
#define	TS_FIXED	1
#define	TS_VARIABLE	0

//***************************************
// keycode definitions

#define K_LBUTT		0x01
#define K_RBUTT		0x02
#define K_MBUTT		0x04

#define K_BACK		0x08
#define K_TAB		0x09
#define K_ENTER		0x0D

#define K_SHIFT		0x10
#define K_CTL		0x11
#define K_MENU		0x12
#define K_ESC		0x1B

#define K_SPACE		0x20
#define K_PGUP		0x21
#define K_PGDN		0x22
#define K_END		0x23
#define K_HOME		0x24
#define K_LFCURS	0x25
#define K_UPCURS	0x26
#define K_RTCURS	0x27
#define K_DNCURS	0x28
#define K_INS		0x2D
#define K_DEL		0x2E

// These are just their ASCII values (uppercase for letters)
// 0-9 are 0x30 - 0x39
// A-Z are 0x41 - 0x5A

#define K_C			0x43
#define K_V			0x56
#define K_X			0x58

#define K_NUMPAD0	0x60
#define K_NUMPAD1	0x61
#define K_NUMPAD2	0x62
#define K_NUMPAD3	0x63
#define K_NUMPAD4	0x64
#define K_NUMPAD5	0x65
#define K_NUMPAD6	0x66
#define K_NUMPAD7	0x67
#define K_NUMPAD8	0x68
#define K_NUMPAD9	0x69
#define K_MULTIPLY	0x6A
#define K_NUMPADADD	0x6B
#define K_SEPARATOR	0x6C
#define K_NUMPADSUB	0x6D
#define K_DECIMAL	0x6E
#define K_NUMPADDIV	0x6F
#define K_F1		0x70
#define K_F2		0x71
#define K_F3		0x72
#define K_F4		0x73
#define K_F5		0x74
#define K_F6		0x75
#define K_F7		0x76
#define K_F8		0x77
#define K_F9		0x78
#define K_F10		0x79
#define K_F11		0x7A
#define K_F12		0x7B

#define K_NUMLOCK	0x90
#define K_SCROLL	0x91

#define K_EQPLUS	0xBB	// =+ key
#define K_UNDERSUB	0xBD	// -_ key
#define K_SLASH		0xBF	// /? key
#define	K_BACKSLASH	0xDC	// \  key

#define K_BACKBUTT  0x8001	// mouse BACK button
#define K_FWDBUTT   0x8002	// mouse FWD button
#define K_WHEEL		0x8003	// mouse wheel


//***************************************
#define BeginText()		_asm(SysBegText)
#define	EndText()		_asm(SysEndText)

//***************************************
// EVENT vector numbers
#define	ON_UNUSED_1			0
#define	ON_UNUSED_2			1
#define	ON_SCREEN_CHANGE	2
#define	ON_ALPHA_KEY_DOWN	3
#define	ON_ALPHA_KEY_UP		4
#define	ON_BEGIN_STARTUP	5
#define ON_END_STARTUP		6
#define	ON_WINDOW_HIDDEN	7
#define	ON_WINDOW_MOVE		8
#define	ON_WINDOW_SIZE		9
#define	ON_POINTER_MOVE		10
#define	ON_KEY_DOWN			11
#define	ON_KEY_UP			12
#define ON_CHAR				13
#define	ON_TIMER			14
#define	ON_IDLE				15
#define	ON_MUSIC_DONE		16
#define	ON_QUERY_QUIT		17
#define	ON_SHUT_DOWN		18

//***************************************
// Function prototypes

void SetCursor(int curs);
void SetProgramTitle(BYTE *str);
void GetWindowPos(WNDPOS *p);
void SetWindowPos(WNDPOS *p, DWORD flag);
void Colorize(HANDLE hBM, long x, long y, long w, long h, long hue, long sat, long lite, long contrast);
HANDLE CreateBM(long w, long h);
void DelBM(HANDLE hBM);
HANDLE LoadBM(BYTE *fname, short dir);
HANDLE LoadMBM(BYTE *fname, short dir, int transx1, int transy1, int transx2, int transy2);
HANDLE LoadLossyBM(BYTE *fname, short dir);
void SaveBM(HANDLE hBM, BYTE *fname, short dir);
void DrawBM(HANDLE dstBM, long dstX, long dstY, HANDLE srcBM, long srcX, long srcY, long srcW, long srcH, long method);
void StretchBM(HANDLE dstBM, long dstX, long dstY, long dstW, long dstH, HANDLE srcBM, long srcX, long srcY, long srcW, long srcH, long method);
void GetBMinfo(HANDLE hBM, BMINFO *pBMI);
void ClipBM(HANDLE hBM, RECT r);
void ClipBMXY(HANDLE hBM, long x1, long y1, long x2, long y2);
RECT GetClipBM(HANDLE hBM);

HANDLE FitFont(BYTE *str, long w, long h, long style, BYTE *charW, long *fontH, long *fontAH);
void RelFont(HANDLE hF);
void SetTextStyle(HANDLE hF, DWORD textclr, DWORD backclr);
long GetTextSize(void *str, long *h, long *ah);
void DrawText(HANDLE hBM, long x, long y, BYTE *str);

DWORD LineStyle(DWORD ls);
void DrawPixel(HANDLE hBM, POINT p, DWORD color);
void DrawPixelXY(HANDLE hBM, long x, long y, DWORD color);
DWORD GetPixel(HANDLE hBM, POINT p);
DWORD GetPixelXY(HANDLE hBM, long x, long y);
void DrawLines(HANDLE hBM, POINT *p, long numpoints, DWORD color);
void Rectangle(HANDLE hBM, RECT r, DWORD fill, DWORD edge);
void RectangleXY(HANDLE hBM, long x1, long y1, long x2, long y2, DWORD fill, DWORD edge);
void GradientRectangle(HANDLE hBM, RECT r, BYTE hue, BYTE minsat, BYTE maxsat, BYTE minlite, BYTE maxlite);
void Oval(HANDLE hBM, RECT r, DWORD fill, DWORD edge);
void OvalXY(HANDLE hBM, long x, long y, long w, long h, DWORD fill, DWORD edge);

DWORD TimeMilli();

#endif	//_KWINLIB_DEFINED
